@@ -3,15 +3,14 @@ from typing import Dict, List, Tuple, Union
 from EasyMCDM.models.MCDM import MCDM
 
 import numpy as np
-from sklearn.preprocessing import StandardScaler
+
 
 class WeightedSum(MCDM):
     
     # Constructor
-    def __init__(self, data : Union[str, np.ndarray, dict], col_sep=',', row_sep='\n', verbose=True, standardized=True):
-        super().__init__(data, col_sep=col_sep, row_sep=row_sep, verbose=verbose)
-        self.standardized = standardized
-        self.scaler = StandardScaler()
+    def __init__(self, data : Union[str, np.ndarray, dict], col_sep=',', row_sep='\n', verbose=True, standardize=False, normalize=True):
+        super().__init__(data, col_sep=col_sep, row_sep=row_sep, verbose=verbose, standardize=standardize, normalize=normalize)
+        self.scaled = standardize or normalize 
 
     # Solve the problem by returning a ranked list of tuple in 0(n) iterations
     def solve(self, pref_indexes: List[int], prefs: List[str], weights: List[float], target='min') -> List[tuple]:
@@ -23,17 +22,16 @@ class WeightedSum(MCDM):
         weights = [weights[i] if prefs[i] == target else weights[i]*-1 for i in range(len(weights))]
 
         # Get only preferred index columns in data
-        matrix = np.asarray([np.array(val)[pref_indexes] for val in self.matrix.values()])
-
-        # Standardize the data
-        if self.standardized == True:
-            matrix = self.scaler.fit_transform(matrix)
+        if ( self.scaled ):
+            matrix = np.asarray([np.array(val)[pref_indexes] for val in self.scaled_matrix])
+        else:
+            matrix = np.asarray([np.array(val)[pref_indexes] for val in self.matrix.values()])
 
         # Weighted Summing
         res = [np.sum(line) for line in np.multiply(matrix[:], [weights for i in self.matrix])]
 
         # Shift Index sort on weighted sum 
-        argres = np.asarray(res).argsort() if target=='min' else (-np.asarray(res)).argsort()
+        argres = np.asarray(res).argsort() if target=='max' else (-np.asarray(res)).argsort()
 
         # Sorting results
         res = [res[index] for index in argres]
@@ -47,7 +45,7 @@ class WeightedSum(MCDM):
         labels = [keys[index] for index in argres]
         scores = [(rank, key, score) for key, score, rank in zip(labels, res, ranks)]
 
-        if "min" in target:
+        if "max" in target:
             scores.reverse()
 
         return scores
\ No newline at end of file
